@@ -158,7 +158,8 @@ void WSHyBiParser::createFrameHeaderFooter(
 
 void WSHyBiParser::read(const char* data, size_t len) {
   ASSERT_BACKGROUND_THREAD()
-  while (len > 0) {
+  bool recur = false;
+  while (len > 0 || recur) {
     // crude check for underflow
     assert(len < 1000000000000000000);
 
@@ -179,6 +180,9 @@ void WSHyBiParser::read(const char* data, size_t len) {
           size_t payloadOffset = frame.headerLength() - startingSize;
           _bytesLeft = frame.payloadLength();
 
+          // Header was consumed, but no payload
+          if (_bytesLeft == 0) recur = true;
+
           _state = InPayload;
           _header.clear();
 
@@ -193,6 +197,8 @@ void WSHyBiParser::read(const char* data, size_t len) {
         break;
       }
       case InPayload: {
+        recur = false;
+
         size_t bytesToConsume = min((uint64_t)len, _bytesLeft);
         _bytesLeft -= bytesToConsume;
         _pCallbacks->onPayload(data, bytesToConsume);
@@ -214,6 +220,12 @@ void WSHyBiParser::read(const char* data, size_t len) {
   }
 }
 
+void WebSocketConnection::startPingTimer() {
+  ASSERT_BACKGROUND_THREAD()
+
+  uv_timer_start(&_pingTimer, pingTimerCallback, 20000, 20000);
+}
+
 bool WebSocketConnection::accept(const RequestHeaders& requestHeaders,
                                  const char* pData, size_t len) {
   ASSERT_BACKGROUND_THREAD()
@@ -223,12 +235,14 @@ bool WebSocketConnection::accept(const RequestHeaders& requestHeaders,
   WebSocketProto_IETF ietf;
   if (ietf.canHandle(requestHeaders, pData, len)) {
     _pParser = new WSHyBiParser(this, new WebSocketProto_IETF());
+    this->startPingTimer();
     return true;
   }
 
   WebSocketProto_HyBi03 hybi03;
   if (hybi03.canHandle(requestHeaders, pData, len)) {
     _pParser = new WSHixie76Parser(this);
+    this->startPingTimer();
     return true;
   }
   return false;
@@ -268,9 +282,16 @@ void WebSocketConnection::sendWSMessage(Opcode opcode, const char* pData, size_t
                            safe_vec_addr(footer), footer.size());
 }
 
+void WebSocketConnection::sendPing() {
+  ASSERT_BACKGROUND_THREAD()
+  assert(_pParser);
+  debug_log("WebSocketConnection::sendPing", LOG_DEBUG);
+  this->sendWSMessage(Ping, NULL, 0);
+}
+
 void WebSocketConnection::closeWS(uint16_t code, std::string reason) {
   ASSERT_BACKGROUND_THREAD()
-  trace("WebSocketConnection::closeWS");
+  debug_log("WebSocketConnection::closeWS", LOG_DEBUG);
 
   switch (_connState) {
   // If we have already sent a close message, do nothing.
@@ -306,12 +327,6 @@ void WebSocketConnection::read(const char* data, size_t len) {
   _pParser->read(data, len);
 }
 
-void WebSocketConnection::read(boost::shared_ptr<std::vector<char> > buf) {
-  ASSERT_BACKGROUND_THREAD()
-  if (_connState == WS_CLOSED) return;
-  read(safe_vec_addr(*buf), buf->size());
-}
-
 void WebSocketConnection::markClosed() {
   ASSERT_BACKGROUND_THREAD()
   _connState = WS_CLOSED;
@@ -341,7 +356,7 @@ void WebSocketConnection::onPayload(const char* data, size_t len) {
 }
 void WebSocketConnection::onFrameComplete() {
   ASSERT_BACKGROUND_THREAD()
-  trace("WebSocketConnection::onFrameComplete");
+  debug_log("WebSocketConnection::onFrameComplete", LOG_DEBUG);
   if (_connState == WS_CLOSED) return;
 
   if (!_header.fin) {
@@ -370,7 +385,7 @@ void WebSocketConnection::onFrameComplete() {
         } else if (_connState == WS_CLOSE_SENT) {
           _connState = WS_CLOSED;
         }
-        
+
         // If we haven't sent a Close frame before, send one now, echoing
         // the callback
         if (_connState != WS_CLOSE_SENT && _connState != WS_CLOSED) {
@@ -404,3 +419,10 @@ void WebSocketConnection::onFrameComplete() {
 
   _payload.clear();
 }
+
+void pingTimerCallback(uv_timer_t* pHandle) {
+  ASSERT_BACKGROUND_THREAD()
+
+  WebSocketConnection* c = reinterpret_cast<WebSocketConnection*>(pHandle->data);
+  c->sendPing();
+}
